@@ -1,3 +1,4 @@
+"""Module containing base action classes used as the foundation of the package. """
 import copy
 import inspect
 import logging
@@ -19,11 +20,12 @@ class Action(InstanceRegistry):
                  **default_kwargs,
                  ):
         """
-        Class for storing and tracking functions and methods to be executed.
+        Base class for storing and tracking actions (functions) to be executed. Effectively this is a wrapper class
+        which provides easy access to attributes relevant to automation (name, signature, kwarg defaults, etc.).
 
-        :param action: action method to execute when called
-        :param default_args: default arguments for the action
-        :param default_kwargs: default kwargs for the action
+        :param action: action (function) to execute when called
+        :param default_args: default arguments for the action. Effectively turns arguments into keyword arguments.
+        :param default_kwargs: default kwargs for the action. Overrides kwarg defaults of the provided action.
         """
         # todo consider implementing action removal
         if self.action_is_registered(action) is True:
@@ -52,7 +54,7 @@ class Action(InstanceRegistry):
 
     @property
     def name(self) -> str:
-        """action method name"""
+        """action (function) name"""
         return self.action.__name__
 
     @property
@@ -235,4 +237,4 @@ class ActionList(MutableSequence):
         self._action_list.insert(
             index,
             action,
-        )
\ No newline at end of file
+        )
