@@ -1,3 +1,4 @@
+"""Module for describing and controlling actions which are configured for execution. """
 import re
 import datetime
 import logging
@@ -32,9 +33,11 @@ class ConfiguredAction(InstanceRegistry, Configuration, Status):
 
         :param action: action to perform (callable)
         :param args: arguments for the action
-        :param configuration_name: optional name for the configuration (for key retrieval)
+        :param configuration_name: optional name for the configuration (for key retrieval). This can be a useful way of
+            differentiating specific configurations of an action if there are multiple configurations for that action.
         :param next_action: optional next action for sequence building
         :param kwargs: keyword arguments for the action
+        :param aliases: list of aliases which refer to the configured action instance
         """
         # todo ability to retrieve the executions of a given configured action
         InstanceRegistry.__init__(self)
@@ -88,7 +91,7 @@ class ConfiguredAction(InstanceRegistry, Configuration, Status):
 
     @property
     def action(self) -> Action:
-        """The action to be tracked"""
+        """the action which is configured for execution"""
         return self._action
 
     @action.setter
@@ -98,7 +101,7 @@ class ConfiguredAction(InstanceRegistry, Configuration, Status):
 
     @property
     def name(self) -> str:
-        """name for the configuration"""
+        """name for the configuration (useful for identifying a specific configuration of an action)"""
         return self._config_name
 
     @name.setter
@@ -373,7 +376,7 @@ class ConfigurableInstance:
 
 
 def ensure_triggered(fn: Callable):
-    """decorator which will do the thing you want"""
+    """a decorator which ensures that the instance has been triggered"""
     @wraps(fn)
     def decorated(self: 'TrackedAction', *args, **kwargs):
         if self.triggered is False:
@@ -420,6 +423,21 @@ class TrackedAction(InstanceRegistry, Configuration, Status):
         self._action_return = None
 
     def __call__(self, *args, **kwargs):
+        """
+        Execution method for a TrackedAction instance. Use extreme care when modifying this method in subclasses.
+        The general order of operations of this method are:
+
+        1. set the triggered flag (prevents multiple executions of the same instance)
+        2. update stored arguments and keyword arguments for the execution of the action
+        3. set the instance as frozen (does not allow updating after this point)
+        4. tracks the start time of the execution
+        5. executes the action, storing the return value
+        6. if an error was encountered, the details of that error are stored
+        7. notes the end time of the execution
+
+        :param args: arguments which will be passed through to the action
+        :param kwargs: keyword arguments which will be passed through to the action
+        """
         if self._triggered is True:
             raise RuntimeError(f'multiple executions of a {self.__class__.__name__} instance is not permitted')
         self._triggered = True
@@ -482,7 +500,7 @@ class TrackedAction(InstanceRegistry, Configuration, Status):
         return self._configuration
 
     def _ensure_unfrozen(fn: Callable):
-        """decorator which will do the thing you want"""
+        """ensures that the instance is not frozen (locked for execution) before performing the specified action"""
         @wraps(fn)
         def decorated(self: 'TrackedAction', *args, **kwargs):
             if self._freeze is True:
@@ -516,7 +534,7 @@ class TrackedAction(InstanceRegistry, Configuration, Status):
 
     @property
     def action_duration(self) -> float:
-        """task duration (s)"""
+        """task total duration (seconds)"""
         if self._time_completed is not None:
             return self._time_completed - self._time_started
 
@@ -599,7 +617,7 @@ class TrackedAction(InstanceRegistry, Configuration, Status):
         """
         warnings.warn(
             'creating TrackedAction instances from ConfiguredAction instances is now the intended method of '
-            'instantiation please update your code to instantiate directly',
+            'instantiation, please update your code to instantiate directly',
             DeprecationWarning,
             stacklevel=2,
         )
@@ -635,17 +653,19 @@ class ThreadedTrackedAction(TrackedAction):
         self._thread: threading.Thread = None
 
     def _executor(self, *args, **kwargs):
-        """target method for the internal thread"""
-        with self._lock:
+        """Target method for the internal thread. Modify this method in subclasses to affect behaviour of execution"""
+        with self._lock:  # allows checking of whether the action is in progress
             super(ThreadedTrackedAction, self).__call__(*args, **kwargs)
 
     def _trigger(self, *args, **kwargs):
         """
-        Triggers the action execution
+        Triggers the action execution. This method was not intended to be overridden. This method also prevents multiple
+        executions of the same instance.
 
         :param args: arguments for the call
         :param kwargs: kwargs for the call
         """
+        # todo double check why this is separate from _executor and whether they can be consolidated
         if self.triggered is True:
             raise SystemError(f'multiple executions of the same {self.__class__.__name__} are not supported')
         logger.debug(f'triggering execution of {self.__str__()}')
@@ -662,7 +682,11 @@ class ThreadedTrackedAction(TrackedAction):
 
     def trigger(self, *args, **kwargs):
         """
-        Triggers the action execution
+        Triggers the action execution. Call order:
+
+        1. _trigger
+        2. _executor
+        3. TrackedAction superclass __call__
 
         :param args: arguments for the call
         :param kwargs: kwargs for the call
@@ -670,6 +694,7 @@ class ThreadedTrackedAction(TrackedAction):
         self._trigger(*args, **kwargs)
 
     def __call__(self, *args, **kwargs):
+        """redirects the call function to operate through _trigger"""
         self._trigger(*args, **kwargs)
 
     @property
