@@ -29,7 +29,7 @@ class InstanceRegistry:
     UUID_PREFIX: str = ''
 
     def __init__(self):
-        """creates a new instance in the registry"""
+        """A class for tracking class instances and retrieving them by UUID"""
         # todo create unittests
         self._uuid: str = str(uuid.uuid4())   # create uuid
         # append instance to registry
