@@ -2,6 +2,7 @@
 Tools for scheduling sequences of actions to execute at prescribed times.
 """
 import os
+import re
 import threading
 import time
 import logging
@@ -31,7 +32,7 @@ class SamplingScheduler(InstanceRegistry):
                  sequence_name: str = None,
                  ):
         """
-        Scheduler for sequences of configured actions.
+        Scheduler for sequences of configured actions. Actions are stored in a configured sequence (sequence attribute).
 
         :param time_deltas: a list of time deltas to initialize the sequence with (passed directly to
             insert_time_point_list)
@@ -127,7 +128,7 @@ class SamplingScheduler(InstanceRegistry):
         return len([tp for tp in self.time_points if tp.triggered is False])
 
     @property
-    def next_time_point(self) -> ActionTimePoint:
+    def next_time_point(self) -> Union[ActionTimePoint, None]:
         """The next time point for triggering"""
         # todo filter out non-action points
         # filter to untriggered time points
@@ -143,6 +144,7 @@ class SamplingScheduler(InstanceRegistry):
     @property
     def estimated_time_until_next_point(self) -> float:
         """estimated time in seconds until next time point"""
+        raise NotImplementedError('not yet implemented')
         return  # todo
 
     @property
@@ -199,6 +201,7 @@ class SamplingScheduler(InstanceRegistry):
                 additional_actions: Iterable[Union[str, callable, ActionTimePoint]] = None,
                 specified_actions: Iterable[Union[str, callable, ActionTimePoint]] = None,
                 wait: bool = None,
+                timepoint_name: str = None,
                 **action_kwargs
                 ) -> ActionTimePoint:
         """
@@ -211,6 +214,7 @@ class SamplingScheduler(InstanceRegistry):
             These actions will be run after the default actions.
         :param specified_actions: actions to run instead of the default actions of the instance
         :param wait: whether to wait for completion of the actions before returning
+        :param timepoint_name: optional name for the timepoint
         :param action_kwargs: keyword arguments for the actions. Keyword arguments will be passed through to their
             respective methods. See ActionTimePoint.trigger for details on kwarg handling
         :return: created ActionTimePoint instance
@@ -230,6 +234,7 @@ class SamplingScheduler(InstanceRegistry):
         manual_time_point = ActionTimePoint(
             time.time() - self._start_time if self._start_time is not None else 0.,
             configuration=action_sequence,
+            name=timepoint_name,
         )
         # store start time if set
         if self._start_time is not None:
@@ -239,6 +244,7 @@ class SamplingScheduler(InstanceRegistry):
             self._time_points,
             manual_time_point
         )
+        self.apply_default_timepoint_names()
         manual_time_point.trigger(
             wait=wait,
             **action_kwargs,
@@ -247,7 +253,11 @@ class SamplingScheduler(InstanceRegistry):
 
     def _sequence_loop(self):
         """
-        Sequence loop that watches time points and triggers actions at the appropriate times
+        Sequence loop that watches time points and triggers actions at the appropriate times. This loop retrieves the
+        next time point of the instance and triggers the sequence of actions associated with the instance.
+
+        If an action is already executing, this loop will block until that previous timepoint is completed before
+        triggering the next time point up.
         """
         while True:
             # if the pause flag has been set
@@ -292,12 +302,28 @@ class SamplingScheduler(InstanceRegistry):
             while self.busy is True:
                 time.sleep(0.5)
 
+    def apply_default_timepoint_names(self, prefix: str = 'Time Point #'):
+        """
+        Applies default timepoint naming across the timepoint list, ensuring consistent naming order.
+
+        :param prefix: prefix to use for naming
+        """
+        name_re = re.compile(f'^{prefix}')
+        counter = 1
+        for tp in self._time_points:
+            # if no name is defined or the pattern matches the prefix, apply default naming
+            #   pattern matching is implemented to not overwrite custom-defined names
+            if tp.name is None or name_re.search(tp.name):
+                tp.name = f'{prefix}{counter}'
+                counter += 1
+
     def insert_time_point(self,
                           time_delta: float = None,
                           position: int = -1,
                           relative_to: str = 'start',
                           shift_subsequent: bool = True,
                           number_of_points: int = 1,
+                          timepoint_name: str = None,
                           additional_actions: Iterable[Union[str, callable, ActionTimePoint]] = None,
                           specified_actions: Iterable[Union[str, callable, ActionTimePoint]] = None,
                           ):
@@ -310,6 +336,8 @@ class SamplingScheduler(InstanceRegistry):
             the time point previous to the insertion point ('previous')
         :param shift_subsequent: whether to shift subsequent time points in the time point list by the time delta
         :param number_of_points: number of points with these specification to add
+        :param timepoint_name: optional name for inserted time point. If multiple points are specified, a counter will
+            be applied.
         """
         action_sequence = self._determine_actions(
             additional_actions=additional_actions,
@@ -338,12 +366,14 @@ class SamplingScheduler(InstanceRegistry):
         self.logger.debug(f'Adding {number_of_points} timepoints at {time_delta} s spacing in position {position}.')
         for i in range(number_of_points):
             delta = (i + 1) * time_delta + delta_offset
+            # create an incremented name if a name was specified
+            tp_name = f'{timepoint_name}{f"#{i + 1}" if number_of_points > 1 else ""}' if timepoint_name else None
             bisect.insort(
                 self._time_points,
                 ActionTimePoint(
                     delta,
                     configuration=action_sequence,
-                    name=f'{delta} s',
+                    name=tp_name,
                 )
             )
 
@@ -351,7 +381,7 @@ class SamplingScheduler(InstanceRegistry):
         if shift_subsequent is True:
             for subsequent_timepoint in self._time_points[i + 1 + position:]:
                 subsequent_timepoint.time_delta += (i + 1) * time_delta
-
+        self.apply_default_timepoint_names()
         # if there the thread is alive and paused (pending more time points), clear flag to resume
         if self._sequence_thread.is_alive() is True and self._sequence_pause.is_set() is False:
             self._sequence_pause.set()
