@@ -102,6 +102,13 @@ class ExecutionSequence(ThreadedTrackedAction):
         return super(ExecutionSequence, self).__str__()
 
     def __call__(self, **kwargs):
+        """
+        Converts the configured actions to a tracked action list and passes those as arguments to the
+        ThreadedTrackedAction call (configured to leverage the execute_sequence_actions operator action).
+
+        :param kwargs:
+        :return:
+        """
         logger.debug(f'{self} sequence execution starting')
         # convert configured actions to tracked actions
         actions = TrackedActionList(*self.configuration.action_sequence)
