@@ -6,6 +6,8 @@ import datetime
 import uuid
 import logging
 import warnings
+import time
+
 from typing import Union, Callable, Iterable
 from .mixin import InstanceRegistry
 from .action import Action, ConfiguredAction, ConfiguredActionList
@@ -25,9 +27,9 @@ class TimePoint(InstanceRegistry):
                  parent_sequence_id: str = None,
                  ):
         """
-        A class for describing and tracking an event time point
+        A class for describing and tracking an event time point relative to some start time.
 
-        :param time_delta: time delta for the time point (relative to some other time)
+        :param time_delta: time delta for the time point (seconds; relative to some other time)
         :param name: convenience name for time point differentiation
         :param description: extended description for differentiation
         :param parent_sequence_id: uuid of parent SamplingScheduler (if applicable)
@@ -133,7 +135,9 @@ class ActionTimePoint(ExecutionSequence, TimePoint):
                  **action_kwargs,
                  ):
         """
-        A class for describing and tracking an event time point where actions (e.g. sampling) are to be triggered
+        A class for describing and tracking an event time point relative to some other time where actions
+        (e.g. a sampling sequence) are to be triggered. The sequence of actions is stored in a ConfiguredSequence
+        instance.
 
         :param time_delta: time delta for the time point (relative to some other time)
         :param name: convenience name for time point differentiation
@@ -164,7 +168,10 @@ class ActionTimePoint(ExecutionSequence, TimePoint):
         return out
 
     def _executor(self, *args, **kwargs):
-        """target method for the internal thread"""
+        """
+        Target method for the internal thread.
+        The ThreadedTrackedAction _executor method is leveraged with the start and completion points noted in the log.
+        """
         logger.info(f'{self} sequence starting')
         super(ActionTimePoint, self)._executor(*args, **kwargs)
         logger.info(f'{self} sequence completed, duration: {self.action_duration}')
@@ -188,7 +195,10 @@ class ActionTimePoint(ExecutionSequence, TimePoint):
                 **kwargs,
                 ):
         """
-        Triggers the actions of the ActionTimePoint. Keyword arguments will be passed through to their respective methods.
+        Triggers (starts execution of) the action sequence of the ActionTimePoint.
+        Keyword arguments will be passed through to their respective action methods by the name of the action.
+        This method executes the built-in __call__ of ExecutionSequence and it will block until the thread of the
+        ExecutionSequence has started.
 
         e.g. if there is a method named "do_this_thing" which expects the keyword argument "value", the expected syntax
         of kwargs would be
@@ -197,6 +207,13 @@ class ActionTimePoint(ExecutionSequence, TimePoint):
                 "do_this_thing": {"value": 42},
             }
 
+        Execution Tree:
+        1. Execution sequence __call__
+        2. ThreadedTrackedAction using operators.execute_sequence_actions action
+        3. Waits until the thread starts (prevents double-triggering
+        4. waits for completion if specified
+
+
         :param wait: whether to wait for completion before returning
         :param kwargs: keyword arguments for the actions. Keyword arguments will be passed through to their respective
             methods.
@@ -204,6 +221,9 @@ class ActionTimePoint(ExecutionSequence, TimePoint):
         """
         # todo refactor to trigger_and_wait (separate so that signatures match)
         self.__call__(**kwargs)
+        # block until triggered flag is set (prevents double-triggering in the same loop)
+        while self._triggered is False:
+            time.sleep(0.001)
         if wait is True:
             self.wait_for_completion()
 
@@ -245,7 +265,9 @@ class ActionTimePoint(ExecutionSequence, TimePoint):
                             name: str = None,
                             ) -> "ActionTimePoint":
         """
-        Creates an instance from a list of configured actions
+        Creates an instance from a list of configured actions. This differs from calling init by accepting
+        an iterable argument containing actions which are used to create the ConfiguredSequence required by
+        init.
 
         :param time_delta: time delta for the time point (relative to some other time)
         :param actions: configured actions or named configurations
